library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.Numeric_STD.all;

library work;
use work.hall_definitions.all;
use work.definitions.all;

-- *********************************************************************************************************** --
-- *
-- *	Overvker hall-sensorene, og sender data videre til kommutatoren. Asynkront begge deler.
-- *	Dersom det skulle vre nskelig, er det mulig  interferere med sendingen, feks ved avansert kommutering.
-- *
-- *	Teller forsinkelsen mellom hvert nye signal fra hall-sensorene, og beregner motorens rotasjonshastighet.
-- *	
-- *	Benytter ekstern divisjonskrets for  utfre deler av beregningene. 
-- *	Ekstern logikk for ressursdeling av divisjonskretsen er ndvendig for flere motorer.
-- *	
-- *	Ekstern modul for generering av interrupt til AVR, og sending av data er ogs ndvendig.
-- *
-- *********************************************************************************************************** --


-- *********************************************************************************************************** --
entity hall_monitor_interface is

	Port(
		sys_clk		: in 	std_logic;	-- Klokke til andre funksjoner.
		rpm_clk		: in 	std_logic;	-- Klokke til telleren. 32,768 kHz.
		reset		: in 	std_logic;	-- Trengs til rpm-estimator

		hall_in		: in	std_logic_vector(2 downto 0);		-- Inngang direkte tilkoblet motorens hall-sensorer.
		hall_out	: out	std_logic_vector(2 downto 0);		-- Porter signalet ut i henhold til kommuteringsvinkel.
		speed		: out	std_logic_vector(speed_width-1 downto 0);-- Tallet vil aldri vre strre enn 13bit. Men br jeg bruke 16bit interface?
		
		division_req	: out	std_logic;	-- Sender foresprsel om  f utfrt divisjon.
		division_ack	: in	std_logic;	-- Tilgang til divisjon innvilget.
		
		dividend_out	: out	std_logic_vector((data_width*2)-1 downto 0);	-- Teller
		divisor_out	: out	std_logic_vector((data_width*1)-1 downto 0);	-- Nevner
		result_in	: in	std_logic_vector((speed_width -1) downto 0);	-- Svar		Redusert i Arbiter.
		
		rpm_counter	: out 	std_logic_vector(counter_width-1 downto 0) := (others => '0')		-- Kun for debugging.
	);

end hall_monitor_interface;
-- *********************************************************************************************************** --



-- *********************************************************************************************************** --
architecture behavioral of hall_monitor_interface is

	signal zone_time	: std_logic_vector(counter_width-1 downto 0) := (others => '0');	-- Signal for  poste teller fra en prosess til en annen.
begin


-- ******************************************************** --
-- *	Porter hall-signalet videre til kommutator:
-- *	Hadde ikke trengt prosess for dette.
-- *	Men denne blir mer kompleks, s snart vinkel-kommutering er p plass!
-- ******************************************************** --
stator_dumper : process (hall_in)
begin
	hall_out <= hall_in;	-- Porter signalet rett ut igjen, til commutator.
end process;
-- ******************************************************** --



-- ******************************************************** --
-- *	RPM_conter
-- ******************************************************** --
rotation_counter : process (rpm_clk, reset)

	variable curr_count : std_logic_vector(counter_width-1 downto 0) := (others => '0'); -- Veldig usikker p strrelsen enn.
	variable last_count : std_logic_vector(counter_width-1 downto 0) := (others => '0'); 
	
	variable curr_hall_zone : std_logic_vector(2 downto 0);-- := "000";	-- Prver  droppe initiering, for  fikse feil.
	variable last_hall_zone : std_logic_vector(2 downto 0);-- := "000";	-- Prver  droppe initiering, for  fikse feil.
begin
	
	----------------------------------------------
	if 	reset = '1' then
		curr_count := (others => '0');
		last_count := (others => '0');
		curr_hall_zone := "000";
		last_hall_zone := "000";
	----------------------------------------------
	elsif 	rising_edge(rpm_clk) then		-- (rpm_clk'event and clk = '1') then
		--------------------------------------
		curr_hall_zone := hall_in;			-- Leser av hall-inngangen, og gjr om til en intern variabel.
		--------------------------------------
		if 	curr_hall_zone = last_hall_zone then	-- Prosessen foretar normal telling:
			------------------------------
			curr_count := curr_count + 1; 		-- Funker dette? Eller burde jeg lage telleren med en int? Er vel best  gjre det med bit vil jeg tro.
			rpm_counter <= curr_count;		-- Bare for  gjre telleren synlig i testbenk.
		--------------------------------------	
		elsif 	curr_hall_zone /= last_hall_zone
		and 	last_hall_zone /= "000" then		-- Prosessen setter igang rpm_estimering, og restetes:
			------------------------------
			last_hall_zone := curr_hall_zone;	-- Denne var visst glemt. Seris feil!
			last_count := curr_count;		-- Tar vare p telleren for videre beregninger.
			curr_count := (others => '0');		-- Klargjr teller for neste periode. Skal jo telle kontinuerlig.
			curr_count := curr_count + 1;	-- Lagt til for  bedre algoritmen!!!!
			zone_time <= last_count;		-- Sender telleren til prosessen som kaller divisjon.
		--------------------------------------
		elsif 	last_hall_zone = "000" then		-- Hndterer initialverdien, og unngr feil rpm_estimat pga oppstart/reset:
			------------------------------	
			last_hall_zone := curr_hall_zone;
		--------------------------------------	
		--elsif curr_hall_zone = "000" 
		--or 	curr_hall_zone = "111" then null;	-- Hndterer eventuelle feiltilstander:
		--------------------------------------
		end if;
	----------------------------------------------
	end if;
	
	
end process;
-- ******************************************************** --

-- ******************************************************** --
-- ******************************************************** --
division_requester : process (zone_time, division_ack) is

	variable last_zone_time : std_logic_vector(counter_width-1 downto 0) := (others => '0');
begin
	----------------------------------------
	if	zone_time /= last_zone_time then
		--------------------------------	
		division_req	<= '1';
		last_zone_time := zone_time;
		
	----------------------------------------
	elsif	division_ack = '1' then
		
		dividend_out <= (others => '0');
		dividend_out(12 downto 0) <= freq_div_6;	-- Har allerede delt p 6.
		divisor_out(counter_width-1 downto 0) <= last_zone_time;
		
		division_req <= '0';
	----------------------------------------
	end if;
end process;
-- ******************************************************** --


-- ******************************************************** --
-- *	Poller resultat-inngangen kontinuerlig.
-- *	Hvis et verdien er ulik sist verdi, s leses denne inn.
-- *
-- *	Et enkelt styfilter, basert p en konstant er implementert her.
-- *	Hvis endringen kun er gjeldende for de X laveste bitene, gitt av konstanten, s ignoreres endringen.
-- *
-- *	Hastigheten sendes kun videre hvis endringen er strre enn den definerte grensen.
-- ******************************************************** --
result_poller : process (result_in) is
	
	constant null_vector : std_logic_vector(speed_width-1 downto 0) := (others => '0');
	variable last_result_in : std_logic_vector(speed_width-1 downto 0) := (others => '0');
begin
	if	result_in(speed_width-1 downto noise_filter) /= last_result_in(speed_width-1 downto noise_filter)
	and 	result_in /= null_vector then
		
		speed <= result_in;	
		last_result_in := result_in;
		
	end if;
end process;
-- ******************************************************** --

end behavioral;
-- *********************************************************************************************************** --