library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.definitions.all;

-- ****************************************************************************************************************** --
entity motor_controller_interface is

	Port(	
		sys_clk		: in std_logic;		-- Systemklokke. Kjrer i frste omgang hyeste tilgjengelige.
		rpm_clk		: in std_logic;		-- 32,768 kHz for beregning av RPM i sanntid.
		pwm_clk		: in std_logic;		-- Forelpig ikke gjort noe beregninger her. Men denne bare trekker ekstra strm ved  vre for hy tror jeg.
		reset		: in std_logic;		-- Global reset av alle registre.
		
		top_hall_in	: in std_logic_vector(2 downto 0); 	-- Tilkobles hall-sensorene.
		
		top_duty_cycle	: in std_logic_vector(7 downto 0); 	-- Pdrag fra regulator (nsket dutycycle)	--throttle	
		top_rotation_dir: in std_logic;				-- nsket rotasjonsretning.	-- rotation_dir

		top_div_req	: out std_logic;
		top_div_ack	: in  std_logic;
		top_dividend	: out std_logic_vector((data_width*2)-1 downto 0);
		top_divisor	: out std_logic_vector((data_width*1)-1 downto 0);
		top_result_in	: in std_logic_vector((speed_width -1) downto 0);
		
		top_speed_out	: out std_logic_vector(speed_width-1 downto 0);
		top_motor_ctrl	: out std_logic_vector(5 downto 0) 	-- Styrer transistorene i H-brua.
	);
end entity motor_controller_interface;
-- ****************************************************************************************************************** --

architecture motor_controller of motor_controller_interface is

	-- ***************** Signal-section: ***************** --
	signal sys_clk_int	: std_logic := '0';		-- Systemklokke. Kjrer i frste omgang hyeste tilgjengelige.
	signal rpm_clk_int	: std_logic := '0';		-- 32,768 kHz tilgjengelig p utviklingskortet.
	signal pwm_clk_int	: std_logic := '0';		-- Forelpig ikke gjort noe beregninger her. Men denne bare trekker ekstra strm ved  vre for hy tror jeg.
	signal reset_int	: std_logic := '0';		-- Global reset av alle registre.
			
	signal hall_sensors_int	: std_logic_vector(2 downto 0) := (others => '0'); 	-- Tilkobles hall-sensorene.
	signal hall_to_com_int	: std_logic_vector(2 downto 0) := (others => '0');	-- Signal fra hall_monitor til commutator.
	signal com_to_mux_int	: std_logic_vector(5 downto 0) := (others => '0');	-- Kommuteringsvektor til h-bridge
	signal pwm_to_mux_int	: std_logic := '0';					-- Fra pwm-modul til mux.
	
	signal duty_cycle_int	: std_logic_vector(7 downto 0) := (others => '0'); 	-- Pdrag fra regulator (nsket dutycycle)
	signal rotation_dir_int	: std_logic := '0';					-- nsket rotasjonsretning.
	
	signal div_req_int	: std_logic := '0';							-- Interface til division_arbiter
	signal div_ack_int	: std_logic := '0';							-- Interface til division_arbiter
	signal dividend_int	: std_logic_vector((data_width*2)-1 downto 0) := (others => '0');	-- Interface til division_arbiter
	signal divisor_int	: std_logic_vector((data_width*1)-1 downto 0) := (others => '0');	-- Interface til division_arbiter
	signal result_int	: std_logic_vector((speed_width-1) downto 0) := (others => '0');	-- Denne reduseres til 13bit i arbiter!
			
--	signal speed_out_int	: std_logic_vector(speed_width-1 downto 0) := (others => '0');	-- Interface til communicator
	signal motor_ctrl_int	: std_logic_vector(5 downto 0) := (others => '0');		-- Styrer transistorene i H-brua.
	
	------------------- Debugging-signals: ------------------
	signal rpm_counter_int	: std_logic_vector(8 downto 0) := (others => '0');		-- For debugging.
	-- ***************** Signal-section: ***************** --

-- ****************************************************************************************************************** --


------------------------------------------------------------------------------------
component hall_monitor_interface
	port(
		sys_clk		: in	std_logic;	-- Egen systemklokke, for  speede opp ting.
		rpm_clk		: in 	std_logic;	-- Klokke til telleren. 32,768 kHz.
		reset		: in 	std_logic;	-- Trengs til rpm-estimator

		hall_in		: in	std_logic_vector(2 downto 0);		-- Inngang direkte tilkoblet motorens hall-sensorer.
		hall_out	: out	std_logic_vector(2 downto 0);		-- Porter signalet ut i henhold til kommuteringsvinkel.
		speed		: out	std_logic_vector(speed_width-1 downto 0);-- Tallet vil aldri vre strre enn 13bit. Men br jeg bruke 16bit interface?
		
		division_req	: out	std_logic;	-- Sender foresprsel om  f utfrt divisjon.
		division_ack	: in	std_logic;	-- Tilgang til divisjon innvilget.
		
		dividend_out	: out	std_logic_vector((data_width*2)-1 downto 0);	-- Teller
		divisor_out	: out	std_logic_vector((data_width*1)-1 downto 0);	-- Nevner
		result_in	: in	std_logic_vector((speed_width -1) downto 0);	-- Svar	-- M vre full lengde! Kuttes ned her.
		
		rpm_counter	: out 	std_logic_vector(8 downto 0) := (others => '0')	-- Kun for debugging.		
	);
end component;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------	
component commutator_interface
	port(
		hall_position	: in	std_logic_vector(2 downto 0);	-- Graykodet soneangivning for rotor.
    		rotation_dir 	: in	std_logic;			-- 1 for hyre, 0 for venstre.
		commutation_out	: out	std_logic_vector(5 downto 0)	-- Kobles til drivtrinnet (h-brua).
	);
end component;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
component commutation_mux_interface
	port(
		clk		: in std_logic;
		reset		: in std_logic;
		commutation_in	: in std_logic_vector(5 downto 0); -- Mitt select-signal.
		pwm_in		: in std_logic;
		commutation_out	: out std_logic_vector(5 downto 0)
	);
end component;
------------------------------------------------------------------------------------

------------------------------------------------------------------------------------
component pwm_fpga
	port(	
		clock 		: in std_logic;				-- Systemklokke spesifikt for PWM.
		reset 		: in std_logic;
		Data_value 	: in std_logic_vector(7 downto 0);	-- Set-port for duty cycle.
		pwm		: out std_logic				-- Ferdig PWM-wave ut.
	);
end component;
------------------------------------------------------------------------------------
	
begin

-- ****************************************************************************** --
--hall1 : entity work.hall_monitor_interface(behavioral)
hall_1 : hall_monitor_interface
	port map(
		sys_clk		=> sys_clk_int,
		rpm_clk		=> rpm_clk_int,
		reset		=> reset_int,

		hall_in		=> hall_sensors_int,
		hall_out	=> hall_to_com_int,
		speed		=> top_speed_out,
		
		division_req	=> div_req_int,
		division_ack	=> div_ack_int,
		
		dividend_out	=> dividend_int,
		divisor_out	=> divisor_int,
		result_in	=> result_int,
		
		rpm_counter	=> rpm_counter_int	-- For debugging.
	);
-- ****************************************************************************** --

-- ****************************************************************************** --
--comm1 : entity atml_work.commutator_interface(commutator)
comm_1 : commutator_interface
	port map(	
		hall_position	=> hall_to_com_int,
    		rotation_dir 	=> rotation_dir_int,
		commutation_out	=> com_to_mux_int
		);
-- ****************************************************************************** --

-- ****************************************************************************** --
--mux1 : entity atml_work.commutation_mux_interface(commutation_mux)
mux_1 : commutation_mux_interface
	port map(	
		clk		=> sys_clk_int,
		reset		=> reset_int,
		commutation_in	=> com_to_mux_int,
		pwm_in		=> pwm_to_mux_int,
		commutation_out	=> motor_ctrl_int
		);
-- ****************************************************************************** --

-- ****************************************************************************** --
--pwm1 : entity atml_work.pwm_fpga(arch_pwm)
pwm_1 : pwm_fpga
	port map(	
		clock 		=> pwm_clk_int,
		reset 		=> reset_int,
		Data_value 	=> duty_cycle_int,
		pwm 		=> pwm_to_mux_int
		);
-- ****************************************************************************** --

	-- ** Signaler ut fra hall-monitor:
	top_div_req 	<= div_req_int;
	div_ack_int	<= top_div_ack;
	top_dividend 	<= dividend_int;
	top_divisor 	<= divisor_int;
	result_int 	<= top_result_in;

	rotation_dir_int<= top_rotation_dir;

	-- ***** Oppdaterer signaler ***** --
	sys_clk_int 	<= sys_clk;
	rpm_clk_int 	<= rpm_clk;
	pwm_clk_int 	<= pwm_clk;
	reset_int	<= reset;
	
--	top_speed_out 	<= speed_out_int;	-- Har koblet direkte
	top_motor_ctrl 	<= motor_ctrl_int;
	
	hall_sensors_int<= top_hall_in;

	duty_cycle_int <= top_duty_cycle;

end architecture motor_controller;
-- ****************************************************************************************************************** --